/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * The test case for the Aggregator.
 * @author Frederic Gardes - EBM WebSourcing
 */
public class AggregatorTest extends AbstractTest {

    private Aggregator aggregator;

    private Document in1;

    private Document in2;

    private Document inTest;

    private String outResult1;

    private String inResult1;

    private String inResult2;

    /**
     * @param name
     */
    public AggregatorTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.aggregator = new Aggregator();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            
            // in xml document 1
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in xml document 2
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorIn2.xml");
            this.in2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in test xml document
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorInTest.xml");
            this.inTest = documentBuilder.parse(inputStream);
            inputStream.close();
            // out result 1 xml document
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorOutResult1.xml");
            this.outResult1 = this.convertStreamToString(inputStream);
            inputStream.close();
            // in result 1 xml document
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorInResult1.xml");
            this.inResult1 = this.convertStreamToString(inputStream);
            inputStream.close();
            // in result 2 xml document
            inputStream = Aggregator.class.getResourceAsStream("/aggregatorInResult2.xml");
            this.inResult2 = this.convertStreamToString(inputStream);
        } finally {
            if(documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if(inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.aggregator = null;
        this.in1 = null;
        this.in2 = null;
        this.inTest = null;
        this.inResult1 = null;
        this.inResult2 = null;
        this.outResult1 = null;
    }

    @Test
    public void testXPATHfactory() throws Exception {
        XPathFactory factory = XPathFactory.newInstance();
        assertNotNull(factory);
        XPath xpath = factory.newXPath();
        assertNotNull(xpath);
        assertNotNull(XPathFactory.newInstance().newXPath());
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDOM() {
        try {
            this.aggregator.init();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(XMLComparator.isEquivalent(
                    this.inResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            this.context.getExchangeList().get(0).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcess2DOM() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange 1
            Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // exchange 2
            exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in2, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(XMLComparator.isEquivalent(
                    this.inResult2,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            this.context.getExchangeList().get(0).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcess2Stream() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange 1
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // exchange 2
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in2, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(XMLComparator.isEquivalent(
                    this.inResult2,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            this.context.getExchangeList().get(0).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessStream() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(XMLComparator.isEquivalent(
                    this.inResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            this.context.getExchangeList().get(0).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.Aggregator#init()}.
     */
    @Test
    public void testInit() {
        try {
            this.aggregator.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        return consumesList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        final List<Param> paramList = new CopyOnWriteArrayList<Param>();
        Param param = new Param();
        param.setName(EIPConstants.TEST);
        param.setValue("boolean(/*[local-name()='PublishContract' and namespace-uri()='http://www.petals.org/ContractService']/*[local-name()='aggregate'])");
        paramList.add(param);
        param = new Param();
        param.setName(EIPConstants.AGGREGATOR_CORRELATION);
        param.setValue("/*[local-name()='PublishContract' and namespace-uri()='http://www.petals.org/ContractService']/*[local-name()='AKContract']/*[local-name()='Type']/text()");
        paramList.add(param);
        return paramList;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOutConsumesWithOut() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOutConsumesWithOut() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOutConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOutConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status or without a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOutConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error statprocessus and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status or without a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOutConsumesWithOut() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 times: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must be on error status
            assertTrue(exchange.isErrorStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must on error status
            assertTrue(exchange.isErrorStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must on active status with an error: it's out of the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getError() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 times: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must be on error status
            assertTrue(exchange.isErrorStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange must be on active status
            assertTrue(exchange.isActiveStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must on error status
            assertTrue(exchange.isErrorStatus());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status
            assertTrue(exchange.isActiveStatus());
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.inTest, null, null, null,
                    this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 time: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange must be on active status and with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithDone() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange can't be on error status or with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() == null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithError() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(true);
            this.context.setFault(false);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 1 time: with the in
            assertEquals(1, exchangeList.size());
            // the exchange must on active status with an error: it's out of the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getError() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Aggregator#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithFault() {
        try {
            this.aggregator.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setError(false);
            this.context.setFault(true);
            // source exchange
            Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.aggregator);
            List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(0, exchangeList.size());
            // the exchange can't be on error status and has got an Out
            assertTrue(exchange.isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    this.outResult1,
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
exchange.getOutMessage().getContent()))));
            // test exchange
            exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.inTest, null, null, null, this.aggregator);
            exchangeList = this.context.getExchangeList();
            // the aggregate exchange is sent 2 times: with the in and with the
            // done status
            assertEquals(2, exchangeList.size());
            // the exchange must be on active status and with a fault
            assertTrue(exchange.isActiveStatus());
            assertTrue(exchange.getFault() != null);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }
}
