/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.xpath.XPathExpressionException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class PatternHelperTest {

    Document patternHelperDocument;

    /**
     * @param name
     */
    public PatternHelperTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            // in xml document
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            inputStream = PatternHelper.class.getResourceAsStream("/patternHelper.xml");
            this.patternHelperDocument = documentBuilder.parse(inputStream);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        this.patternHelperDocument = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.PatternHelper#evaluateXPathCondition(java.lang.String, org.w3c.dom.Node, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testEvaluateXPathCondition() {
        String expression = null;
        boolean evaluation = false;
        try {
            expression = "/*[namespace-uri()='http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb' and local-name()='EnvoiESB']/*[namespace-uri()='' and local-name()='informationsGF']/*[namespace-uri()='' and local-name()='idEmetteur']='GF'";
            evaluation = PatternHelper.evaluateXPathCondition(expression,
                    this.patternHelperDocument.getDocumentElement());
            assertTrue(evaluation);
            expression = "/*[namespace-uri()='http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb' and local-name()='EnvoiESB']/*[namespace-uri()='' and local-name()='informationsGF']/*[namespace-uri()='' and local-name()='idEmetteur']='MDEL'";
            evaluation = PatternHelper.evaluateXPathCondition(expression,
                    this.patternHelperDocument.getDocumentElement());
            assertFalse(evaluation);
        } catch (final XPathExpressionException e) {
            fail("unable to evaluate: " + e.getMessage());
        }
    }
}
