/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.patterns.mock.ExchangeContextMock;
import org.ow2.petals.se.eip.patterns.mock.ExchangeImplMock;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class SplitterTest extends AbstractAggregatorPatternTest {

    private Document in1;

    private Document in2;

    private Document in3;

    private Document in4;

    private Document in5;

    private DataHandler attachment1;

    private DataHandler attachment2;

    /**
     * @param name
     */
    public SplitterTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.abstractForkerPattern = new Splitter();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();

            // in 1
            inputStream = Splitter.class.getResourceAsStream("/splitterIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in 2
            inputStream = Splitter.class.getResourceAsStream("/splitterIn2.xml");
            this.in2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in 3
            inputStream = Splitter.class.getResourceAsStream("/splitterIn3.xml");
            this.in3 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in 4
            inputStream = Splitter.class.getResourceAsStream("/splitterAttachment1.xml");
            this.in4 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in 5
            inputStream = Splitter.class.getResourceAsStream("/splitterAttachment2.xml");
            this.in5 = documentBuilder.parse(inputStream);
            inputStream.close();
            // attachment 1
            URL url = Splitter.class.getResource("/splitterAttachment1.xml");
            this.attachment1 = new DataHandler(url);
            inputStream.close();
            // attachment 2
            url = Splitter.class.getResource("/splitterAttachment2.xml");
            this.attachment2 = new DataHandler(url);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.in1 = null;
        this.in2 = null;
        this.in3 = null;
        this.in4 = null;
        this.in5 = null;
        this.attachment1 = null;
        this.attachment2 = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Splitter#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDOM() {
        try {
            this.abstractForkerPattern.init();
            this.createContext("false");
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.abstractForkerPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in2),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in3),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(1)
                            .getInMessage().getContent()))));
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Splitter#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessStream() {
        try {
            this.abstractForkerPattern.init();
            this.createContext("false");
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.abstractForkerPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in2),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in3),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(1)
                            .getInMessage().getContent()))));
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Splitter#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessAttachment() {
        try {
            this.abstractForkerPattern.init();
            this.createContext("true");
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            final Exchange exchange = this.processAttachmentTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null, this.abstractForkerPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            final String stringResult1 = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(exchangeList.get(0).getInMessage().getContent()));
            final String stringResult2 = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(exchangeList.get(1).getInMessage().getContent()));
            assertTrue(XMLPrettyPrinter.prettyPrint(this.in4).equals(stringResult1)
                    && XMLPrettyPrinter.prettyPrint(this.in5).equals(stringResult2)
                    || XMLPrettyPrinter.prettyPrint(this.in4).equals(stringResult2)
                    && XMLPrettyPrinter.prettyPrint(this.in5).equals(stringResult1));
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.Splitter#init()}.
     */
    @Test
    public void testInit() {
        try {
            this.abstractForkerPattern.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    private Exchange processAttachmentTest(final URI pattern, final Role role,
            final ExchangeStatus status, final Document in, final Document out,
            final Document fault, final Exception exception, final AbstractPattern abstractPattern)
            throws MessagingException, PEtALSCDKException {
        final SourceKind sourceKind = new SourceKind();
        sourceKind.setStreamKind();
        final ExchangeImplMock exchange = this.createExchange(sourceKind, pattern, role, status,
                null, in, out, fault, exception);
        exchange.setCheckRole(false);
        exchange.addInMessageAttachment("one", this.attachment1);
        exchange.addInMessageAttachment("two", this.attachment2);
        exchange.setCheckRole(true);
        abstractPattern.logger = this.logger;
        abstractPattern.process(exchange, this.context);
        return exchange;
    }

    private void createContext(final String attachment) {
        this.context = new ExchangeContextMock(new ConfigurationExtensions(
                this.getParamList(attachment)));
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test"));
        consumesList.add(consumes);
        return consumesList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return null;
    }

    private List<Param> getParamList(final String attachment) {
        final List<Param> paramList = new CopyOnWriteArrayList<Param>();
        Param param = new Param();
        param.setName(EIPConstants.TEST);
        param.setValue("/*[local-name()='PublishContract' and namespace-uri()='http://www.petals.org/ContractService']/*[local-name()='AKContract']");
        paramList.add(param);
        param = new Param();
        param.setName(EIPConstants.ATTACHMENT_MODE);
        param.setValue(attachment);
        paramList.add(param);
        return paramList;
    }

}
