/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns.mock;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.process.async.AsyncContext;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ExchangeContextMock implements ExchangeContext {

    private List<Consumes> consumeList;

    private Logger logger;

    private List<Exchange> exchangeList;

    private ConfigurationExtensions configurationExtensions;

    private boolean error;

    private boolean fault;

    /**
     * Set the error
     * 
     * @param error
     *            the error to set
     */
    public void setError(final boolean error) {
        this.error = error;
    }

    /**
     * Set the fault
     * 
     * @param fault
     *            the fault to set
     */
    public void setFault(final boolean fault) {
        this.fault = fault;
    }

    /**
     * Get the exchangeList
     * 
     * @return the exchangeList
     */
    public List<Exchange> getExchangeList() {
        return this.exchangeList;
    }

    /**
     * @param consumeList
     * @param configurationExtensions
     */
    public ExchangeContextMock(final ConfigurationExtensions configurationExtensions) {
        super();
        this.configurationExtensions = configurationExtensions;
        this.logger = Logger.getLogger(ExchangeContextMock.class.getName());
        this.exchangeList = new ArrayList<Exchange>();
    }

    /**
     * Set the consumeList
     * 
     * @param consumeList
     *            the consumeList to set
     */
    public void setConsumeList(final List<Consumes> consumeList) {
        this.consumeList = consumeList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#createConsumeExchange(org.ow2.petals
     * .component.framework.jbidescriptor.generated.Consumes)
     */
    public Exchange createConsumeExchange(final Consumes consumes) throws MessagingException,
            PEtALSCDKException {
        final MEPConstants mep = this.retrievePattern(consumes);
        final Exchange exchange = this.createConsumeExchange(consumes, mep);
        return exchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#createConsumeExchange(org.ow2.petals
     * .component.framework.jbidescriptor.generated.Consumes,
     * org.ow2.petals.component.framework.api.Message.MEPConstants)
     */
    public Exchange createConsumeExchange(final Consumes consumes, final MEPConstants mep)
            throws MessagingException, PEtALSCDKException {
        final Exchange exchange = this.createExchange(mep);
        this.setOperationFromConsumes(exchange, consumes);
        this.setMessageExchangePropertiesFromConsumes(exchange, consumes);
        this.setExchangeDestinationForConsumes(exchange, consumes);
        return exchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.ExchangeContext#getExtensions()
     */
    public ConfigurationExtensions getExtensions() {
        return this.configurationExtensions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.ExchangeContext#getLogger()
     */
    public Logger getLogger() {
        return this.logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#getSUConsumes(javax.jbi.servicedesc
     * .ServiceEndpoint)
     */
    public List<Consumes> getSUConsumes(final ServiceEndpoint endpoint) {
        return this.consumeList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#send(org.ow2.petals.component.framework
     * .api.message.Exchange)
     */
    public void send(final Exchange exchange) throws MessagingException {
        this.exchangeList.add(exchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#sendAsync(org.ow2.petals.component
     * .framework.api.message.Exchange,
     * org.ow2.petals.component.framework.process.async.AsyncContext)
     */
    public void sendAsync(final Exchange exchange, final AsyncContext AsyncContext)
            throws MessagingException {
        this.exchangeList.add(exchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.ExchangeContext#sendSync(org.ow2.petals.component
     * .framework.api.message.Exchange)
     */
    public boolean sendSync(final Exchange exchange) throws MessagingException {
        if (exchange instanceof ExchangeImpl) {
            final MessageExchange messageExchange = ((ExchangeImpl) exchange).getMessageExchange();
            if (messageExchange instanceof MessageExchangeMock) {
                ((MessageExchangeMock) messageExchange).setRole(MessageExchange.Role.PROVIDER);
            }
        }
        if (exchange.isActiveStatus()) {
            if (exchange.isInOnlyPattern()) {
                if (this.error) {
                    exchange.setError(new Exception("The context set an error"));
                } else {
                    exchange.setDoneStatus();
                }
            } else if (exchange.isRobustInOnlyPattern()) {
                if (this.error) {
                    exchange.setError(new Exception("The context set an error"));
                } else {
                    if (this.fault) {
                        final Fault fault = exchange.createFault();
                        try {
                            fault.setContent(SourceUtil.createSource(
                                    "<content>The context set a fault</content>"));
                        } catch (final PEtALSCDKException e) {
                            throw new MessagingException(e);
                        }
                        exchange.setFault(fault);
                    } else {
                        exchange.setDoneStatus();
                    }
                }
            } else if (exchange.isInOutPattern()) {
                try {
                    if (this.fault) {
                        final Fault fault = exchange.createFault();
                        fault.setContent(SourceUtil.createSource(
                                "<content>The context set a fault</content>"));
                        exchange.setFault(fault);
                    } else {
                        exchange.getOutMessage().setContent(
                                SourceUtil.createSource(
                                        "<content>The context set an out</content>"));
                    }
                } catch (final PEtALSCDKException e) {
                    throw new MessagingException(e);
                }
            } else if (exchange.isInOptionalOutPattern()) {
                if (this.error) {
                    exchange.setError(new Exception("The context set an error"));
                } else {
                    try {
                        if (this.fault) {
                            final Fault fault = exchange.createFault();
                            fault.setContent(SourceUtil.createSource(
                                    "<content>The context set a fault</content>"));
                            exchange.setFault(fault);
                        } else {
                            exchange.getOutMessage().setContent(
                                    SourceUtil.createSource(
                                            "<content>The context set an out</content>"));
                        }
                    } catch (final PEtALSCDKException e) {
                        throw new MessagingException(e);
                    }
                }
            }
        }
        // activate the cache storage, like the processor
        if (exchange instanceof ExchangeImpl) {
            final MessageExchange messageExchange = ((ExchangeImpl) exchange).getMessageExchange();
            if (messageExchange instanceof MessageExchangeMock) {
                ((MessageExchangeMock) messageExchange).setRole(MessageExchange.Role.CONSUMER);
            }
        }
        this.exchangeList.add(exchange);
        return true;
    }

    /**
     * Retrieve the MEP from the consumer node extensions.
     * 
     * @param consumes
     *            The consumer node of the SU JBI descriptor.
     * @return MEP specified in extensions, or <code>null</code> if no MEP
     *         pattern specified.
     * @throws MessagingException
     *             MEP pattern specified in consumes extension is not valid.
     */
    private final MEPConstants retrievePattern(final Consumes consumes) throws MessagingException {
        MEPConstants mep = null;

        if (consumes.getMep() != null) {
            if (MEPType.IN_ONLY.equals(consumes.getMep())) {
                mep = MEPConstants.IN_ONLY_PATTERN;
            } else if (MEPType.ROBUST_IN_ONLY.equals(consumes.getMep())) {
                mep = MEPConstants.ROBUST_IN_ONLY_PATTERN;
            } else if (MEPType.IN_OUT.equals(consumes.getMep())) {
                mep = MEPConstants.IN_OUT_PATTERN;
            } else if (MEPType.IN_OPTIONAL_OUT.equals(consumes.getMep())) {
                mep = MEPConstants.IN_OPTIONAL_OUT_PATTERN;
            } else {
                throw new MessagingException("MEP pattern '" + consumes.getMep().value()
                        + "' is not valid");
            }
        }

        return mep;
    }

    /**
     * Create an exchange implementation matching the given pattern.
     * 
     * @param mep
     *            a specific MEP pattern
     * @return an exchange implementation matching the given pattern
     * @throws MessagingException
     *             The message exchange can't be created or no pattern specified
     *             in extensions
     * @throws PEtALSCDKException
     *             The current Transaction has failed to return its status
     */
    private Exchange createExchange(final MEPConstants mep) throws MessagingException,
            PEtALSCDKException {
        final MessageExchange exchange = new MessageExchangeMock(mep.value(), Role.CONSUMER);
        return new ExchangeImpl(exchange);
    }

    /**
     * Set operation to call if exists in extensions. If no extension can be
     * found or if operation value is "" or missing, operation isn't set.
     * 
     * @param exchange
     *            a {@link MessageExchange}, cannot be null
     * @param extensions
     */
    private final void setOperationFromConsumes(final Exchange exchange, final Consumes consumes) {

        if (consumes.getOperation() != null) {
            exchange.setOperation(consumes.getOperation());
        }
    }

    /**
     * Set message exchange properties from extensions. All extensions key
     * starting with "org.ow2.petals" are automatically added to message
     * exchange properties.
     * 
     * @param exchange
     *            a {@link Exchange}, cannot be null
     * @param extensions
     * @throws PEtALSCDKException
     */
    private final void setMessageExchangePropertiesFromConsumes(final Exchange exchange,
            final Consumes consumes) {

        if (consumes.getExchangeProperties() != null) {
            final ConfigurationExtensions configurationExtensions = new ConfigurationExtensions(
                    consumes.getExchangeProperties().getExchangeProperty());

            if (configurationExtensions != null) {
                for (final String name : configurationExtensions.keySet()) {
                    exchange.setProperty(name, configurationExtensions.get(name));
                }
            }
        }
    }

    /**
     * Fill the exchange with the service information (itf,src, and ep) from the
     * consume extension
     * 
     * @param exchange
     */
    private void setExchangeDestinationForConsumes(final Exchange exchange, final Consumes consumes) {

        // Retrieve possible target endpoint
        ServiceEndpoint ep = null;
        if (consumes.getEndpointName() != null && consumes.getServiceName() != null) {
            final QName[] interfaces = new QName[1];
            interfaces[0] = new QName("http://petals.ow2.org", "interface");
            ep = new ServiceEndpointMock(interfaces, consumes.getServiceName(),
                    consumes.getEndpointName());
        }

        // set exchange information
        exchange.setInterfaceName(consumes.getInterfaceName());
        exchange.setService(consumes.getServiceName());
        exchange.setEndpoint(ep);

        if (consumes.getLinkType() != null) {
            exchange.setProperty(Exchange.PROPERTY_PROVIDER_LINKTYPE, consumes.getLinkType()
                    .value());
        }
    }
}
